use std::{fs::create_dir_all, io::Cursor, path::PathBuf};

use directories::ProjectDirs;
use iced_video_player::Video;
use ::image::{DynamicImage, ImageReader};
use iced::{gradient, widget::container, Color};

use crate::{LauncherBackground, PossibleGames, VideoBackground, ORGANIZATION, PRODUCT};

use super::img_utils::round_image;

pub fn get_game_background(game: &PossibleGames, make_video: bool) -> Result<LauncherBackground, std::io::Error> {
    let proj_dirs = ProjectDirs::from("com", ORGANIZATION, PRODUCT).unwrap();
    let background_path = get_asset_file(&proj_dirs, game, "background")?;

    if background_path.extension().unwrap().to_str().unwrap() == "mp4" {
        let first_frame_path = get_asset_file(&proj_dirs, game, "first_frame_image")?;
        let first_frame = ImageReader::new(Cursor::new(std::fs::read(first_frame_path)?))
            .with_guessed_format()
            .unwrap()
            .decode()
            .unwrap();

        if !make_video {
            return Ok(LauncherBackground::Video(VideoBackground {
                first_frame,
                video: None
            }));
        }
        match Video::new(&url::Url::from_file_path(background_path).unwrap()) {
            Ok(video) => {
                let video_object = VideoBackground {
                    first_frame,
                    video: Some(video)
                };

                Ok(LauncherBackground::Video(video_object))
            },
            Err(err) => {
                eprintln!("{:#?}", err);
                Ok(LauncherBackground::Video(VideoBackground {
                    first_frame,
                    video: None
                }))
            },
        }
    } else {
        let img = ImageReader::new(Cursor::new(std::fs::read(background_path)?))
            .with_guessed_format()
            .unwrap()
            .decode()
            .unwrap();
        Ok(LauncherBackground::Image(img))
    }
}

fn get_asset_file(proj_dirs: &ProjectDirs, game: &PossibleGames, identifier: &str) -> Result<PathBuf, std::io::Error> {
    let game_dir = match game {
        PossibleGames::WutheringWaves => proj_dirs.data_dir().join("kuro/wuwa"),
        PossibleGames::ZenlessZoneZero => proj_dirs.data_dir().join("hoyoverse/zzz"),
        PossibleGames::HonkaiStarRail => proj_dirs.data_dir().join("hoyoverse/hsr"),
        PossibleGames::GenshinImpact => proj_dirs.data_dir().join("hoyoverse/gi"),
    };
    
    if create_dir_all(&game_dir).is_err() {
        return Err(std::io::Error::new(
            std::io::ErrorKind::NotFound,
            format!("Game directory does not exist: {:?}", game_dir)
        ));
    }
    
    let entries = std::fs::read_dir(&game_dir)?;
    
    for entry in entries {
        let entry = entry?;
        let file_name = entry.file_name().into_string().unwrap_or_default();
        
        if file_name.starts_with(format!("{}_", identifier).as_str()) {
            return Ok(entry.path());
        }
    }
    
    Err(std::io::Error::new(
        std::io::ErrorKind::NotFound,
        format!("No background file found in {:?}", game_dir)
    ))
}

pub fn get_game_splash_dynamic_image(game: &PossibleGames) -> Option<DynamicImage> {
    let proj_dirs = ProjectDirs::from("com", ORGANIZATION, PRODUCT).unwrap();
    let file_path = get_asset_file(&proj_dirs, game, "splash");

    if let Ok(path) = file_path {
        let data_cursor = Cursor::new(std::fs::read(path).unwrap());
        
        Some(ImageReader::new(data_cursor)
        .with_guessed_format()
        .unwrap()
        .decode()
        .unwrap())
    } else {None}
}

pub fn get_game_icon_dynamic_image(game: &PossibleGames) -> DynamicImage {
    let proj_dirs = ProjectDirs::from("com", ORGANIZATION, PRODUCT).unwrap();
    let file_data: &[u8] = match game {
        PossibleGames::WutheringWaves => include_bytes!("../../resources/wutheringwaves-icon.png"),
        PossibleGames::ZenlessZoneZero => &std::fs::read(get_asset_file(&proj_dirs, game, "icon").unwrap()).unwrap(),
        PossibleGames::HonkaiStarRail => &std::fs::read(get_asset_file(&proj_dirs, game, "icon").unwrap()).unwrap(),
        PossibleGames::GenshinImpact => &std::fs::read(get_asset_file(&proj_dirs, game, "icon").unwrap()).unwrap(),
    };

    let data_cursor = Cursor::new(file_data);
        
    let img = ImageReader::new(data_cursor)
    .with_guessed_format()
    .unwrap()
    .decode()
    .unwrap()
    .resize(128, 128, ::image::imageops::FilterType::Lanczos3);

    round_image(img)
        .unwrap()
}

fn rad(deg: f32) -> f32 {
    deg * std::f32::consts::PI / 180.0
}

pub fn style_container(direction: f32, use_gradient: bool) -> container::Style {
    let angle = rad(direction);
    let gradient: Option<iced::Background> = if use_gradient {            
        Some(gradient::Linear::new(angle)
        .add_stop(0.0, Color::from_rgba8(0, 0, 0, 0.0))
        .add_stop(0.75, Color::from_rgba8(0, 0, 0, 0.75)).into())
    } else {None};
    container::Style {
        text_color: Color::from_rgba8(255, 255, 255, 1.0).into(),
        background: gradient,
        ..container::Style::default()
    }
}